@@ -65,6 +65,7 @@ class FakeModule:
         if self.station_poller:
             return self.station_poller(sta, snmp_client, http_client)
         return sta
+    
 
 class FakeClient:
     def __init__(self, host=None):
@@ -163,6 +164,7 @@ class TestAP(unittest.TestCase):
     def test_airtime_calc3(self):
         """Test the airtime calculations in case no conn_time available from Stations. We use AP's uptime instead"""
         ap = Ap(self.ctx, 'TestNE', 'Test Element', 'site', None, None, 'ap', None)
+        self._await(ap.log_ne_update()) # Resolve race condition with NEU message
         ap.module = FakeModule()
         ap.errctx = getattr(ap.module, 'errctx', ErrorContext())
         ap.network_element_update_msg = getattr(ap.module, 'network_element_update_msg', NetworkElementUpdate(errctx=ap.errctx))
@@ -200,7 +202,7 @@ class TestAP(unittest.TestCase):
             link_tx_airtime = ne_u.data.interfaces[0].radios[0].streams[0].links[n].tx_airtime
             rx_link_rate = float(ne_u.data.interfaces[0].radios[0].streams[0].links[n].rx_link_rate)
             tx_link_rate = float(ne_u.data.interfaces[0].radios[0].streams[0].links[n].tx_link_rate)
-
+    
             self.assertEqual(link_rx_airtime, 6000000 * 8 * 100 / (rx_link_rate * 60))
             self.assertEqual(link_tx_airtime, 3000000 * 8 * 100 / (tx_link_rate * 60))
             total_rx_airtime += link_rx_airtime
@@ -429,3 +431,151 @@ class TestAP(unittest.TestCase):
         # port rf level airtime equals sum of stations' airtime for Mikrotik models (both directions)
         self.assertEqual(ne_u.data.interfaces[1].radios[0].rx_airtime, None)
         self.assertEqual(ne_u.data.interfaces[1].radios[0].tx_airtime, None)
+
+
+    def test_airtime_calc10(self):
+        """Test the airtime calculations when the peers have the same AP counter_source"""
+        df = self.datafiles.get(('cambium-epmp.epmp3000.ePMP 3000.4.3.2.01', 'snmp'))
+        ne = self._await(self.reg.set('TestNE', {'name': 'Test Element', 'site': 'My Site', 'path': df.path}))
+        ap = Ap(self.ctx, ne.id, ne.name, ne.site, ne.snmp_ne, ne.snmp_ne_v3, ne.ne_type, ne.holdoff)
+        self._await(ap.log_ne_update())
+        
+        # Poll 1: Set to AP counter_source
+        self._await(ap.poll())
+        msg = ap.module.network_element_update_msg
+        msg.peers[0].counter_source = "AP"
+
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets = 1000
+            msg.data.interfaces[1].links[n].out_octets = 1000
+
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        msg.data.interfaces[1].radios[0].streams[0].links[0].connected_time += 60 
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets += 6000000
+            msg.data.interfaces[1].links[n].out_octets += 3000000
+
+        # Poll 2: Keep AP as counter_source
+        msg.data.uptime += 60
+        msg.peers[0].counter_source = "AP"
+        self._await(ap.log_ne_update())
+
+        self._await(ap.poll())
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        self.assertIsNotNone(msg.data.interfaces[1].radios[0].rx_airtime)
+        self.assertIsNotNone(msg.data.interfaces[1].radios[0].tx_airtime)
+
+
+    def test_airtime_calc11(self):
+        """Test the airtime calculations when the peers have the same CPE counter_source"""
+        df = self.datafiles.get(('cambium-epmp.epmp3000.ePMP 3000.4.3.2.01', 'snmp'))
+        ne = self._await(self.reg.set('TestNE', {'name': 'Test Element', 'site': 'My Site', 'path': df.path}))
+        ap = Ap(self.ctx, ne.id, ne.name, ne.site, ne.snmp_ne, ne.snmp_ne_v3, ne.ne_type, ne.holdoff)
+        self._await(ap.log_ne_update())
+        
+        # Poll 1: Set to CPE counter_source
+        self._await(ap.poll())
+        msg = ap.module.network_element_update_msg
+        msg.peers[0].counter_source = "CPE"
+
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets = 1000
+            msg.data.interfaces[1].links[n].out_octets = 1000
+
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        msg.data.interfaces[1].radios[0].streams[0].links[0].connected_time += 60 
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets += 6000000
+            msg.data.interfaces[1].links[n].out_octets += 3000000
+
+        # Poll 2: Keep as CPE counter_source
+        msg.data.uptime += 60
+        msg.peers[0].counter_source = "CPE"
+        self._await(ap.log_ne_update())
+
+        self._await(ap.poll())
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        self.assertIsNotNone(msg.data.interfaces[1].radios[0].rx_airtime)
+        self.assertIsNotNone(msg.data.interfaces[1].radios[0].tx_airtime)
+
+
+    def test_airtime_calc12(self):
+        """Test the airtime calculations when the peers AP and CPE have different counter_source"""
+        df = self.datafiles.get(('cambium-epmp.epmp3000.ePMP 3000.4.3.2.01', 'snmp'))
+        ne = self._await(self.reg.set('TestNE', {'name': 'Test Element', 'site': 'My Site', 'path': df.path}))
+        ap = Ap(self.ctx, ne.id, ne.name, ne.site, ne.snmp_ne, ne.snmp_ne_v3, ne.ne_type, ne.holdoff)
+        self._await(ap.log_ne_update())
+        
+        # Poll 1: Set to AP counter_source
+        self._await(ap.poll())
+        msg = ap.module.network_element_update_msg
+        msg.peers[0].counter_source = "AP"
+
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets = 1000
+            msg.data.interfaces[1].links[n].out_octets = 1000
+
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        msg.data.interfaces[1].radios[0].streams[0].links[0].connected_time += 60 
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets += 6000000
+            msg.data.interfaces[1].links[n].out_octets += 3000000
+
+        # Poll 2: Change to CPE counter_source
+        msg.data.uptime += 60
+        msg.peers[0].counter_source = "CPE"
+        self._await(ap.log_ne_update())
+
+        self._await(ap.poll())
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        self.assertIsNone(msg.data.interfaces[1].radios[0].rx_airtime)
+        self.assertIsNone(msg.data.interfaces[1].radios[0].tx_airtime)
+
+
+    def test_airtime_calc13(self):
+        """Test the airtime calculations when the peers CPE and AP have different counter_source"""
+        df = self.datafiles.get(('cambium-epmp.epmp3000.ePMP 3000.4.3.2.01', 'snmp'))
+        ne = self._await(self.reg.set('TestNE', {'name': 'Test Element', 'site': 'My Site', 'path': df.path}))
+        ap = Ap(self.ctx, ne.id, ne.name, ne.site, ne.snmp_ne, ne.snmp_ne_v3, ne.ne_type, ne.holdoff)
+        self._await(ap.log_ne_update())
+        
+        # Poll 1: Set to CPE counter_source
+        self._await(ap.poll())
+        msg = ap.module.network_element_update_msg
+        msg.peers[0].counter_source = "CPE"
+
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets = 1000
+            msg.data.interfaces[1].links[n].out_octets = 1000
+
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        msg.data.interfaces[1].radios[0].streams[0].links[0].connected_time += 60 
+        for n in range(len(msg.data.interfaces[1].links)):
+            msg.data.interfaces[1].links[n].in_octets += 6000000
+            msg.data.interfaces[1].links[n].out_octets += 3000000
+
+        # Poll 2: Change to AP counter_source
+        msg.data.uptime += 60
+        msg.peers[0].counter_source = "AP"
+        self._await(ap.log_ne_update())
+
+        self._await(ap.poll())
+        self.wait_for_ap_poll()
+        self._await(ap.log_ne_update())
+
+        self.assertIsNone(msg.data.interfaces[1].radios[0].rx_airtime)
+        self.assertIsNone(msg.data.interfaces[1].radios[0].tx_airtime)
